# Copyright (c) 2007-2008 Forest Bond.
# This file is part of the pytagsfs software package.
#
# pytagsfs is free software; you can redistribute it and/or modify it under the
# terms of the GNU General Public License version 2 as published by the Free
# Software Foundation.
#
# A copy of the license has been included in the COPYING file.

from pytagsfs.subspat import ModifiedIfElseSubstitutionPattern, PatternError
from pytagsfs.exceptions import UnrepresentableRealPath, InvalidArgument

class PathParser(object):
    meta_parser = None
    format_string = None
    substitution_pattern = None

    def __init__(self, meta_parser, format_string):
        self.meta_parser = meta_parser
        self.format_string = format_string

        self.substitution_pattern = ModifiedIfElseSubstitutionPattern(
          format_string)

    def parse(self, real_path):
        d = self.meta_parser.parse(real_path)
        try:
            return self.substitution_pattern.fill(d)
        except KeyError:
            raise UnrepresentableRealPath(real_path)

    def apply(self, real_path, fake_path):
        try:
            d = self.substitution_pattern.split(fake_path)
        except PatternError:
            raise InvalidArgument
        return self.meta_parser.apply(real_path, d)
