# Copyright (c) 2007-2008 Forest Bond.
# This file is part of the pytagsfs software package.
#
# pytagsfs is free software; you can redistribute it and/or modify it under the
# terms of the GNU General Public License version 2 as published by the Free
# Software Foundation.
#
# A copy of the license has been included in the COPYING file.

import sys, os, locale

from optparse import OptionParser

import sclapp
from sclapp import CriticalError

from pytagsfs.subspat import IfElseSubstitutionPattern, PatternError
from pytagsfs.metaparser import MUTAGEN_FIELD_MAPPING
from pytagsfs.util import SimpleMutagenFile
from pytagsfs import __version__ as version

def print_tags(filenames):
    failed_filenames = [ ]
    for filename in filenames:
        print filename

        try:
            f = SimpleMutagenFile(filename)
            if f is None:
                raise IOError('Invalid file: %s' % filename)
        except Exception, e:
            sclapp.printCritical(unicode(e))
            failed_filenames.append(filename)
            print
            continue

        print f.pprint()
        print

    if failed_filenames:
        return 1
    return 0

def update_tags(pattern, set_tags, remove_tags, filenames):
    substitution_pattern = None
    if pattern:
        substitution_pattern = IfElseSubstitutionPattern(pattern)

    failed_filenames = [ ]

    for filename in filenames:
        print filename

        if substitution_pattern:
            try:
                pattern_tags = substitution_pattern.split(filename)
            except PatternError, e:
                sclapp.printCritical(unicode(e))
                failed_filenames.append(filename)
                continue
        else:
            pattern_tags = { }

        try:
            f = SimpleMutagenFile(filename)
            if f is None:
                raise IOError('Invalid file: %s' % filename)
        except Exception, e:
            sclapp.printCritical(unicode(e))
            failed_filenames.append(filename)
            continue

        try:
            for k in pattern_tags:
                f[MUTAGEN_FIELD_MAPPING[k]] = pattern_tags[k]

            for tag in set_tags:
                f[tag] = set_tags[tag]

            for tag in remove_tags:
                try:
                    del f[tag]
                except KeyError:
                    sclapp.printWarning(u'No such tag: %s' % tag)

        except Exception, e:
            sclapp.printCritical(unicode(e))
            failed_filenames.append(filename)
            continue

        f.save()

    if failed_filenames:
        sclapp.printCritical('Failed to process the following files:')
    for filename in failed_filenames:
        sclapp.printCritical(filename)

    if failed_filenames:
        return 1
    return 0

def parse_set_tags(set_exprs):
    set_tags = { }
    for expr in set_exprs:
        try:
            k, v = expr.split('=')
        except (TypeError, ValueError):
            raise CriticalError(1, 'Invalid set expression: %s' % expr)
        set_tags[k] = v
    return set_tags

@sclapp.main_function
def main(argv):
    parser = OptionParser(
      usage = '%prog [options] {file} [file...]',
      version = '%%prog version %s' % version,
    )
    parser.add_option('--pattern')
    parser.add_option('--set', action = 'append', default = [ ])
    parser.add_option('--remove', action = 'append', default = [ ])
    values, args = parser.parse_args(argv[1:])

    if len(args) < 1:
        raise CriticalError(1, 'Too few arguments.')

    encoding = locale.getpreferredencoding()

    pattern = values.pattern
    set_tags = parse_set_tags(values.set)
    remove_tags = values.remove
    filenames = args

    if not (pattern or set_tags or remove_tags):
        return print_tags(filenames)
    else:
        return update_tags(pattern, set_tags, remove_tags, filenames)
