# Copyright (c) 2007-2008 Forest Bond.
# This file is part of the pytagsfs software package.
#
# pytagsfs is free software; you can redistribute it and/or modify it under the
# terms of the GNU General Public License version 2 as published by the Free
# Software Foundation.
#
# A copy of the license has been included in the COPYING file.

class SourceTreeMonitor(object):
    def start(self, source_tree_rep):
        '''Start monitoring the source tree.'''

    def stop(self):
        '''Stop monitoring the source tree; clean up.'''

    def fileno(self):
        '''
        Return a file descriptor that can be used to poll for new events.  If
        unsupported, raise NotImplementedError; this is the default behavior,
        so this method can be left unimplemented.
        '''

    def process_events(self):
        '''Process pending events.  Do not block.'''

    def add_source_dir(self, real_path):
        '''
        Monitor source directory ``real_path``.  Fail silently if directory
        can't be monitored due to normal runtime conditions (don't mask bugs).
        '''

    def remove_source_dir(self, real_path):
        '''
        Stop monitoring source directory ``real_path``.  Fail silently if
        directory was not previously being monitored.
        '''

    def add_source_file(self, real_path):
        '''
        Monitor source file ``real_path``.  Fail silently if file can't be
        monitored due to normal runtime conditions (don't mask bugs).
        '''

    def remove_source_file(self, real_path):
        '''
        Stop monitor source file ``real_path``.  Fail silently if file was
        not previously being monitored.
        '''

    def supports_threads(self):
        '''
        Return True if source tree monitor is thread-safe.  The default
        implementation always returns False.
        '''
        return False

    def supports_writes(self):
        '''
        Return True if source tree monitor supports updates due to writing in
        mount tree.  The default implementation always returns False.
        '''
        return False
