from threading import RLock

from pytagsfs.sourcetreemon import SourceTreeMonitor
from pytagsfs.util import (
  nonatomic,
  attr_named,
)

ADD = object()
REMOVE = object()
UPDATE = object()

class DeferredSourceTreeMonitor(SourceTreeMonitor):
    event_queue = None
    dstm_lock = None

    def __init__(self, *args, **kwargs):
        self.dstm_lock = RLock()
        super(DeferredSourceTreeMonitor, self).__init__(*args, **kwargs)
        self.event_queue = {}

    @nonatomic(attr_named('dstm_lock'))
    def add_cb(self, path, *args):
        if path not in self.event_queue:
            self.event_queue[path] = []
        self.event_queue[path].append((ADD, path) + args)

    @nonatomic(attr_named('dstm_lock'))
    def remove_cb(self, path, *args):
        if path not in self.event_queue:
            self.event_queue[path] = []
        self.event_queue[path].append((REMOVE, path) + args)

    @nonatomic(attr_named('dstm_lock'))
    def update_cb(self, path, *args):
        t = (UPDATE, path) + args
        if path not in self.event_queue:
            self.event_queue[path] = [t]
        else:
            if (self.event_queue[path][-1] != t):
                self.event_queue[path].append(t)

    @nonatomic(attr_named('dstm_lock'))
    def finish_processing(self):
        while self.event_queue:
            for path in self.event_queue.keys():
                try:
                    event = self.event_queue[path].pop(0)
                except (IndexError, KeyError):
                    del self.event_queue[path]
                    continue
                if event[0] == ADD:
                    super(DeferredSourceTreeMonitor, self).add_cb(*(event[1:]))
                elif event[0] == REMOVE:
                    super(DeferredSourceTreeMonitor, self).remove_cb(
                      *(event[1:]))
                elif event[0] == UPDATE:
                    super(DeferredSourceTreeMonitor, self).update_cb(
                      *(event[1:]))
                else:
                    raise ValueError('unknown action %s' % str(event[0]))

    @nonatomic(attr_named('dstm_lock'))
    def process_events(self):
        super(DeferredSourceTreeMonitor, self).process_events()
        self.finish_processing()
